% Description: Funkcia vypota hodnoty odhadov elementrnych zloiek kostnho
%              tkaniva vo zvolenej oblasti zujmu (ROI), z dvoj-energetickch 
%              CT akvizci. Funkcia taktie obsahuje vnoren funkciu
%              (get_BMD, komparatvna metda) pre alternatvne vypotanie 
%              priemernho odhadu BMD z jedno-energetickch CT dt identickho 
%              tomografickho rezu modifikcie s tukom pre vizulne porovnanie 
%              vstupov obidvoch metd.
% Input:    ID          = ID pacienta [double] (3,6,13,14,17,28,29),                    VZOR: 13
%           H           = VYIA energetick hladina keV [double] (150,100,80,60,50),   VZOR: 100
%           L           = NIIA energetick hladina keV [double] (40,50,60,80,100),    VZOR: 80
%           slice       = index CT rezu [char],                                         VZOR: 'I320'
%           filePath    = cesta k pacientskm sborom CT akvizci spolu s .xlsx tabukou
%                         (export_sVNC.xlsx) podrobnejch zdrojov informci [char]    VZOR: 'C:\User\Documents\Martin_Liska_BP_priloha\CT_akvizicie'
% Output:   output      = tabuka priemernch odhadov hodnt ROI z DVOJ-energetickch CT dt [table] 
%                         (BMD, V_TB, V_F, V_T, V_BM, V_C, c_TBV, c_Ca)
% -------------------------------------------------------------------
% Authors:  Martin Liska
%           Jiri Chmelik
% ===================================================================
function [ output ] = BMD_dual_E_estimation( ID, H, L, slice, filePath )
%% NATANIE SBOROV/METADT
% logick opercie pre automatick vyhadanie prieinku a zskanie
% informci z tabuky export_sVNC.xlsx
H = num2str(H);
L = num2str(L);
refFile = readtable([ filePath '/' 'export_sVNC.xlsx' ]); warning('off','MATLAB:table:ModifiedAndSavedVarnames')    % Natanie .xlsx sboru s pacientskmi referenciami
pDirectory = char( refFile.PatientDirectory( find ( refFile.PatientID==ID, 1, 'first') ) );
energiesID = refFile.EnergyDirectory(refFile.PatientID==ID);
eDirectory_H = char(energiesID(ismember(refFile.Energy_keV_(refFile.PatientID==ID), H)));
eDirectory_L = char(energiesID(ismember(refFile.Energy_keV_(refFile.PatientID==ID), L)));
eDirectory_K = char(energiesID(ismember(refFile.Energy_keV_(refFile.PatientID==ID), 'kCT')));

% automatick natanie zvolench CT akvizci (DICOM)
CT_H = dicomread([ filePath '\' pDirectory '\' eDirectory_H '\' slice ]);       % Natanie DICOM sboru viac-energetickej CT akvizcie pre interaktvnu prcu (ROI)
CT_L = dicomread([ filePath '\' pDirectory '\' eDirectory_L '\' slice ]);       % Natanie DICOM sboru CT akvizcie niej energie
CT_K = dicomread([ filePath '\' pDirectory '\' eDirectory_K '\' slice ]);       % Natanie DICOM sboru CT akvizcie vyej energie

% natanie metadt DICOM sborov
info_K = dicominfo([ filePath '\' pDirectory '\' eDirectory_K '\' slice ]);
info_H = dicominfo([ filePath '\' pDirectory '\' eDirectory_H '\' slice ]);
info_L = dicominfo([ filePath '\' pDirectory '\' eDirectory_L '\' slice ]);

% zskanie informci o pohlav pacienta
if info_K.PatientSex == 'F'
    sex = 'Female' ;
else
    sex = 'Male' ;
end
% zskanie informci o veku pacienta
age = info_K.PatientAge(2:3) ;
%% INTERAKTVNE VYTVORENIE ROI
f1 = figure('Name', 'Interaktvne vytvorenie ROI');
imshow (double(CT_K),[]) ;
title(['Multi-Energy CT (ID: ' num2str(ID) ', CT-slice: ' slice ')' ])
set(gcf, 'Position', get(0,'Screensize'));                                  % Fullscreen figure
message = sprintf('Dvoj-klikom ukonte zadvanie ROI');
uiwait(msgbox(message));
h = drawellipse('Color','g'); waitfordoubleclick                            % ROI potvrdte dvojklikom
center = round(h.Center);
vert = round(h.Vertices);
ax = round(h.SemiAxes);
BW = createMask(h);                                                         % Eliptick maska ROI
close(figure(f1))
%% PREDSPRACOVANIE DT
CT_H = (double(CT_H) * info_H.RescaleSlope) + info_H.RescaleIntercept;      % klovanie obrazovch dt do HU: U=m*SV+b
CT_L = (double(CT_L) * info_L.RescaleSlope) + info_L.RescaleIntercept;

filt = fspecial('disk', 3);                                                 % Defincia parametrov rozmazvacieho filtru
CT_H = roifilt2(filt,CT_H, BW);                                             % Pouitie rozmazvaciejho filtru
CT_L = roifilt2(filt,CT_L, BW);
%% FUNKCIA BMD_dual_E_calculation PRE CEL OBRAZOV MATICU
[ BMD, V_TB, V_F, V_T, V_BM, V_C, c_TBV, c_Ca ] = calculation( CT_H, CT_L, str2double(H), str2double(L), BW);
%% FIGURES PRIEMERNHO ODHADU BMD V ROI A FAREBN REPREZENTCIA
BMD_m = mean(BMD(BW));                                                      % Priemern odhad BMD v oblasti zujmu [mg/cm^3]

% figure - priemern odhad BMD
X1 = mat2gray(CT_K);
rgbImage = cat(3, X1, X1, X1);                                              % Vytvorenie RGB obrazu
Image = rgbImage;
for i=1:length(vert)                                                        % Vytvorenie vizulneho obrysu ROI
    Image(vert(i,2),vert(i,1),:)=[0,1,0];
end
text = insertText(Image,[center(1) center(2)-ax(1)],[ num2str(round(BMD_m,4)) ' mg/cm^3' ],'FontSize',14,...
'BoxColor','yellow','BoxOpacity',0.4,'TextColor','white','AnchorPoint', 'CenterBottom' ); % Vizulna defincia priemernho odhadu BMD vo figure
figure('Name', 'Priemern odhad BMD z DVOJ-ENERGETICKCH CT dt')
imshow(text,[])
title(['BMD (ROI-mean, ' sex ' (' age '), ID: ' num2str(ID) ', CT-slice: ' slice ')' ])

% figure - farebn reprezentcia
BMD_c = BMD;
BMD_c(BMD_c<min(BMD_c(BW))) = min(BMD_c(BW));                               % Preklovanie hodnot celej matice (mxn) min. a max. hodnotami ROI
BMD_c(BMD_c>max(BMD_c(BW))) = max(BMD_c(BW));                               % Preklovanie hodnot celej matice (mxn) min. a max. hodnotami ROI
BMD_c = uint32(255 * mat2gray(BMD_c));
rgbImage = ind2rgb(BMD_c, flipud(hot(256)));                                % Pridelenie farebnej reprezentcie celej matici

redChannel = rgbImage(:, :, 1);                                             % Extrakcia ervenho, zelenho a modrho kanlu z farebnej reprezentcie
greenChannel = rgbImage(:, :, 2);
blueChannel = rgbImage(:, :, 3);

outputImageR = rescale(double(CT_K));                                       % Vytvorenie novch farebnch kanlov pre plochu mimo ROI
outputImageG = rescale(double(CT_K));
outputImageB = rescale(double(CT_K));

outputImageR(BW) = redChannel(BW);                                          % Transformcia farebnch kanlov v oblasti zujmu (ROI) do pozadia CT snmok
outputImageG(BW) = greenChannel(BW);
outputImageB(BW) = blueChannel(BW);
outputRGBImage = cat(3, outputImageR, outputImageG, outputImageB);          % Spojenie farebnch kanlov do jednho RGB obrazu
figure('Name', 'Farebn reprezentcia z DVOJ-ENERGETICKCH CT dt')
imshow(outputRGBImage);
colormap(flipud(hot))
title(['BMD (ROI-color, ' sex ' (' age '), ID: ' num2str(ID) ', CT-slice: ' slice ')' ])
c=colorbar('Ticks',[0,0.25,0.5,0.75,1]...
    ,'TickLabels',{num2str(round((min(BMD(BW))/5)*2,2)),num2str(round((max(BMD(BW))/5)*2,2)),num2str(round(max(BMD(BW))/2,2)),...
    num2str(round(max(BMD(BW))/5*4,2)),num2str(round(max(BMD(BW)),2))});    % Pridanie colorbar so zadefinovanmi hodnotami
c.Label.String = '[mg/cm^3]';
axis on;
%% OUTPUT (table)
BMD = BMD_m;
Elements = {'BMD';'V_TB';'V_F';'V_T';'V_BM';'V_C';'c_TBV';'c_Ca'};
Estimation = [round(BMD,3); round(V_TB,3); round(V_F,3); round(V_T,3); round(V_BM,3); round(V_C,3); round(c_TBV); round(c_Ca,3)];
Units = {'mg/cm^3';'cm^3';'cm^3';'cm^3';'cm^3';'cm^3';'mg/cm^3';'mg/cm^3'};
output = table(Estimation, Units,'RowNames',Elements);

%% ========================================================================
%  RIEENIE A UKKA KOMPARATVNEJ METDY (pre rchle porovnanie vsledkov)
%  ========================================================================
answer = questdlg('Pokraova vo vpote odhadu BMD komparatvnej metdy?','Komparatvna Metda','no','Ukoni','no');
    switch answer
        case 'Ukoni'
            disp('Odhad BMD a elementrnych zloiek z dvoj-energetickch CT dt:')
        case 'no'
            disp('Odhad BMD a elementrnych zloiek z dvoj-energetickch CT dt:')
%% FIGURES FUNKCIE get_BMD (ROI-muscle, ROI-fat)
% interaktvne ROI - muscle
f1 = figure('Name', 'Interaktvne vytvorenie ROI svalovho tkaniva');
imshow (double(CT_K),[]) ;
title(['Multi-Energy CT - muscle (ID: ' num2str(ID) ', CT-slice: ' slice ')' ])
set(gcf, 'Position', get(0,'Screensize'));                                  % Fullscreen figure
message = sprintf('Dvoj-klikom ukonte zadvanie ROI-MUSCLE');
uiwait(msgbox(message));
h = drawellipse('Color','r'); waitfordoubleclick
maska_muscle = createMask(h);                                               % Eliptick maska
close(figure(f1))

% interaktvne ROI - fat
f1 = figure('Name', 'Interaktvne vytvorenie ROI tukovho tkaniva');
imshow (double(CT_K),[]) ;
title(['Multi-Energy CT - fat (ID: ' num2str(ID) ', CT-slice: ' slice ')' ])
set(gcf, 'Position', get(0,'Screensize'));                                  % Fullscreen figure
message = sprintf('Dvoj-klikom ukonte zadvanie ROI-FAT');
uiwait(msgbox(message));
h = drawellipse('Color','y'); waitfordoubleclick
maska_fat = createMask(h);                                                  % Eliptick maska
close(figure(f1))

%% PREDSPRACOVANIE DT
CT_K_t = (double(CT_K) * info_K.RescaleSlope) + info_K.RescaleIntercept;
CT_K_t = roifilt2(filt,CT_K_t,BW);
muscle_mean = roifilt2(filt,CT_K_t,maska_muscle);                           % Pouitie rozmazvacieho filtru
muscle_mean = mode(round(muscle_mean(maska_muscle),1));                     % Medin odhadu ROI-muscle
fat_mean = roifilt2(filt,CT_K_t,maska_fat);                                 % Pouitie rozmazvacieho filtru
fat_mean = mode(round(fat_mean(maska_fat),1));                              % Medin odhad ROI-fat

BMD_get_BMD = get_BMD( CT_K_t, muscle_mean, fat_mean, false ) ;             % Volanie funkcie get_BMD
BMD_m_get_BMD = median(BMD_get_BMD(BW));                                    % Medin odhadu BMD v ROI

%% FIGURES FUNKCIE get_BMD (analgia s predolmi figures)
% figure - priemern odhad BMD (get_BMD)
text = insertText(Image,[center(1) center(2)-ax(1)],[ num2str(round(BMD_m_get_BMD,4)) ' mg/cm^3' ],'FontSize',14,...
'BoxColor','yellow','BoxOpacity',0.4,'TextColor','white','AnchorPoint', 'CenterBottom' );
figure('Name', 'Priemern odhad BMD z JEDNO-ENERGETICKCH CT dt')
imshow(text,[])
title(['BMD (ROI-mean, ' sex ' (' age '), ID: ' num2str(ID) ', CT-slice: ' slice ', COMPARATIVE)' ])

% figure - farebn reprezentcia
BMD_c_get=BMD_get_BMD;
BMD_c_get(BMD_c_get<min(BMD_c_get(BW))) = min(BMD_c_get(BW));               % Preklovanie hodnot celej matice min. a max. hodnotami ROI
BMD_c_get(BMD_c_get>max(BMD_c_get(BW))) = max(BMD_c_get(BW));               % Preklovanie hodnot celej matice min. a max. hodnotami ROI
BMD_c_get = uint32(255 * mat2gray(BMD_c_get));
rgbImage_get = ind2rgb(BMD_c_get, flipud(hot(256)));                        % Pridelenie farebnej reprezentcie celej matici

redChannel_get = rgbImage_get(:, :, 1);                                     % Extrakcia ervenho, zelenho a modrho kanlu z farebnej reprezentcie
greenChannel_get = rgbImage_get(:, :, 2);
blueChannel_get = rgbImage_get(:, :, 3);

outputImageR_get = rescale(double(CT_K));                                   % Vytvorenie novch farebnch kanlov pre vstup
outputImageG_get = rescale(double(CT_K));
outputImageB_get = rescale(double(CT_K));

outputImageR_get(BW) = redChannel_get(BW);                                  % Transformcia farebnch kanlov v oblasti zujmu (ROI)
outputImageG_get(BW) = greenChannel_get(BW);
outputImageB_get(BW) = blueChannel_get(BW);
outputRGBImage_get = cat(3, outputImageR_get, outputImageG_get, outputImageB_get);  % Spojenie farebnch kanlov do RGB obrazu
figure('Name', 'Farebn reprezentcia z JEDNO-ENERGETICKCH CT dt')
imshow(outputRGBImage_get);
colormap(flipud(hot))
title(['BMD (ROI-color, ' sex ' (' age '), ID: ' num2str(ID) ', CT-slice: ' slice ', COMPARATIVE)' ])
c=colorbar('Ticks',[0,0.25,0.5,0.75,1]...
    ,'TickLabels',{num2str(round((min(BMD_get_BMD(BW))/5)*2,2)),num2str(round((max(BMD_get_BMD(BW))/5)*2,2)),num2str(round(max(BMD_get_BMD(BW))/2,2)),...
    num2str(round(max(BMD_get_BMD(BW))/5*4,2)),num2str(round(max(BMD_get_BMD(BW)),2))});  % Pridanie colorbar so zadefinovanmi hodnotami
c.Label.String = '[mg/cm^3]';
axis on;
    end
end